
' file access
Const ForReading = 1
Const ForWriting = 2
Const ForAppending = 8

' file names
Const sHostDb = "hosts"           ' source hosts file
Const sNullDb = "null.db"         ' null db filename
Const sConfig = "extra.conf"      ' BIND config filename
Const sNullIP = "0.0.0.0"         ' null IP address

' objects
Dim   fso, finp, fout
dim   dict

' other stuff
Dim   lTotRec, lRecNo


' =========== MAIN
  Message "*"
  Message "Starting HOSTS conversion program..."
  Message "Input file name....: " & sHostDB
  Message "BIND config file...: " & sConfig
  Message "BIND null zone.....: " & sNullDb
  Message "Blocked hosts IP...: " & sNullIP
  If ConvertHosts() Then
    Message "Conversion successfully completed."
  End If  
  Message "*"
' =========== END


Sub Message(sText)
  If sText <> "*" Then
    WScript.Echo Now() & ": " & sText
  Else
    WScript.Echo ""
  End If
End Sub

Function InitProgram()

  InitProgram = False

  Set fso = CreateObject("Scripting.FileSystemObject")
  Set dict = CreateObject("Scripting.Dictionary")

  lTotRec = 0
  lRecNo = 0

  ' Hosts (input)
  Message "Opening " & sHostDb & " input file..."
  Set finp = fso.OpenTextFile(sHostDb, ForReading, False)
  If finp.AtEndOfStream = True Then
    MsgBox "Source (hosts) file is empty, can't continue"
    finp.Close
    Exit Function
  End If

  ' null zone file database
  Message "Initializing " & sNullDb & " null zone file..."
  Set fout = fso.OpenTextFile(sNullDb, ForWriting, True)
  With fout
    .WriteLine("")
    .WriteLine("; NULL Zone File for Ad Servers")
    .WriteLine(";")
    .WriteLine("$TTL 36000")
    .WriteLine("@               in      soa     localhost. postmaster.localhost. (")
    .WriteLine("                                2002110101      ;serial")
    .WriteLine("                                3600            ;refresh")
    .WriteLine("                                1800            ;retry")
    .WriteLine("                                604800          ;expiration")
    .WriteLine("                                3600 )          ;minimum")
    .WriteLine("")
    .WriteLine(";")
    .WriteLine(";  Zone NS records")
    .WriteLine(";")
    .WriteLine("")
    .WriteLine("@                      NS	localhost.")
    .WriteLine("")
    .WriteLine("		                    A	127.0.0.1")
    .WriteLine("*	                  IN	A	" & sNullIp)
    .WriteLine("")
  End With
  fout.Close

  ' extra config file for hosts blocking
  Message "Opening " & sConfig & " output file..."
  Set fout = fso.OpenTextFile(sConfig, ForWriting, True)
  With fout
    .WriteLine("")
    .WriteLine("// --- blocked hosts/domains")
    .WriteLine("// --- " & Now())
    .WriteLine("")
  End With

  InitProgram = True
End Function

Sub ResetProgram()
  On Error Resume Next
  fout.Close
  finp.Close
  Set fout = Nothing
  Set finp = Nothing
  Set fso = Nothing
  Set dict = Nothing
End Sub

Function ReadHost(fp)
  Dim sRecord, nPos

  ReadHost = ""
  sRecord = fp.ReadLine
  sRecord = Replace(sRecord, Chr(9), Chr(32))
  sRecord = Replace(sRecord, Chr(10), Chr(32))
  sRecord = Replace(sRecord, Chr(13), Chr(32))
  sRecord = Trim(sRecord)
  If (Mid(sRecord, 1, 1) = "#") Or (Len(sRecord) < 1) Then
    Exit Function
  End If

  nPos = Instr(sRecord, Chr(32))
  If nPos > 0 Then
    sRecord = Trim(Mid(sRecord, nPos + 1))
  End If
  nPos = Instr(sRecord & Chr(32), Chr(32))
  ReadHost = Trim(Mid(sRecord, 1, nPos -1))
End Function

Sub WriteHost(sHost, fp)
  Dim sRecord

  sRecord = "zone " & Chr(34) & sHost & Chr(34) 
  sRecord = sRecord & " { type master; notify no; file " & Chr(34) & sNullDb & Chr(34) & "; };"
  fp.WriteLine sRecord
End Sub

Function MakeHostKey(sHost)
  Dim sKey, iChar

  For iChar = Len(sHost) To 1 Step -1
    sKey=sKey & Mid(sHost, iChar, 1)
  Next 
  MakeHostKey = "[" & sKey & "]"
End Function

Function ConvertHosts()
  Dim sHost, sKey, iHost, oItems

  ' initialize
  Message "Initializing the script..."
  ConvertHosts = False
  If InitProgram() = False Then
    Exit Function
  End If

  lRecNo = 0
  lTotRec = 0

  ' load hosts
  Message "Loading the hosts list..."
  While finp.AtEndOfStream = False
    lTotRec = lTotRec + 1
    sHost = ReadHost(finp)
    If Len(sHost) > 0 Then
      sKey = MakeHostKey(sHost)
      If dict.Exists(sKey) = False Then
        lRecNo = lRecNo + 1
        dict.Add sKey, sHost
      End If
    End If
  Wend
  Message lTotRec & " record(s) loaded, " & lRecNo & " host record(s)"
  lTotRec = lRecNo

  ' write dns zone file
  Message "Writing hosts infos to the config file..."
  oItems = dict.Items
  For iHost = 0 To dict.Count -1
    sHost = oItems(iHost)
    WriteHost sHost, fout
    lRecNo = lRecNo + 1
  Next  

  ' finalize
  Message "Releasing objects and exiting..."
  ResetProgram
  If lRecNo > 0 Then
    ConvertHosts = True
  End If
End Function

